from analogai.deepthink.thinker import DeepThinker
from analogai.foundation.modules.fact.fact_source import FactSource, SourceType


thinker = DeepThinker()


if __name__ == "__main__":
    thinker.remember(
        "Paris is the capital of France.",
        source=FactSource(SourceType.WEBSITE, "https://en.wikipedia.org/wiki/Paris"),
    )
    response = thinker.chat("What is the capital of France?")
    print(response if response else "(no response)")






# if __name__ == "__main__":
#     while True:
#         message = input("Message: ")
#         result = thinker.chat(message)
#         print(result if result else "(no response)")
