from __future__ import annotations

from analogai.deepthink.messaging.identity.user_agent_resolver import UserAgentResolver


class UserAgentResolverFactory:
    def __init__(
        self,
        default_user_id: str,
        default_agent_id: str,
        default_authority: float = 1.0,
    ):
        self._default_user_id = default_user_id
        self._default_agent_id = default_agent_id
        self._default_authority = default_authority

    def create(self) -> UserAgentResolver:
        return UserAgentResolver(
            default_user_id=self._default_user_id,
            default_agent_id=self._default_agent_id,
            default_authority=self._default_authority,
        )
