from __future__ import annotations

RELAXED_SEARCH_NOTICE = ""


def get_relaxed_search_notice() -> str:
    return RELAXED_SEARCH_NOTICE


def prepend_relaxed_notice(text: str) -> str:
    if not text:
        return text
    notice = get_relaxed_search_notice()
    if not notice:
        return text
    cleaned = str(text).strip()
    if not cleaned:
        return text
    if cleaned.lower().startswith(notice.lower()):
        return cleaned
    return f"{notice}{cleaned}"
